@@ -24,6 +24,35 @@
 #include "private.h"
 #include "missing.h"
 
+/*
+ * RFC 5114 - Section 2.4 192-bit Random ECP Group
+ */
+#define P192_CURVE_P { 0xFFFFFFFFFFFFFFFFull, 0xFFFFFFFFFFFFFFFEull, \
+			0xFFFFFFFFFFFFFFFFull }
+#define P192_CURVE_GX { 0xF4FF0AFD82FF1012ull, 0x7CBF20EB43A18800ull, \
+			0x188DA80EB03090F6ull }
+#define P192_CURVE_GY { 0x73F977A11E794811ull, 0x631011ED6B24CDD5ull, \
+			0x07192B95FFC8DA78ull }
+#define P192_CURVE_N { 0x146BC9B1B4D22831ull, 0xFFFFFFFF99DEF836ull, \
+			0xFFFFFFFFFFFFFFFFull }
+#define P192_CURVE_B { 0xFEB8DEECC146B9B1ull, 0x0FA7E9AB72243049ull, \
+			0x64210519E59C80E7ull }
+
+static const struct l_ecc_curve p192 = {
+	.name = "secp192r1",
+	.ike_group = 25,
+	.tls_group = 19,
+	.ndigits = 3,
+	.g = {
+		.x = P192_CURVE_GX,
+		.y = P192_CURVE_GY,
+		.curve = &p192
+	},
+	.p = P192_CURVE_P,
+	.n = P192_CURVE_N,
+	.b = P192_CURVE_B,
+};
+
 /*
  * RFC 5114 - Section 2.6 256-bit Random ECP Group
  */
@@ -92,6 +121,7 @@ static const struct l_ecc_curve p384 = {
 static const struct l_ecc_curve *curves[] = {
 	&p384,
 	&p256,
+	&p192,
 };
 
 /* Returns supported IKE groups, sorted by the highest effective key size */
@@ -547,8 +577,24 @@ LIB_EXPORT struct l_ecc_point *l_ecc_point_from_data(
 		if (!_ecc_compute_y(curve, p->y, p->x))
 			goto failed;
 
+		/*
+		 * This is determining whether or not to subtract the Y
+		 * coordinate from P. According to ANSI X9.62 an even Y should
+		 * be prefixed with 02 (BIT0) and an odd Y should be prefixed
+		 * with 03 (BIT1). If this is not the case, subtract Y from P.
+		 *
+		 * ANSI X9.62
+		 * 4.3.6 Point-to-Octet-String Conversion
+		 *
+		 * 2. If the compressed form is used, then do the following:
+		 *     2.1. Compute the bit ~Yp . (See Section 4.2.)
+		 *     2.2. Assign the value 02 to the single octet PC if ~Yp
+		 *          is 0, or the value 03 if ~Yp is 1.
+		 *     2.3. The result is the octet string PO = PC || X
+		 */
+
 		sub = secure_select(type == L_ECC_POINT_TYPE_COMPRESSED_BIT0,
-					!(p->y[0] & 1), p->y[0] & 1);
+					p->y[0] & 1, !(p->y[0] & 1));
 
 		_vli_mod_sub(tmp, curve->p, p->y, curve->p, curve->ndigits);
 
@@ -790,9 +836,17 @@ LIB_EXPORT struct l_ecc_scalar *l_ecc_scalar_new(
 	return NULL;
 }
 
+LIB_EXPORT struct l_ecc_scalar *l_ecc_scalar_clone(const struct l_ecc_scalar *s)
+{
+	if (!s)
+		return NULL;
+
+	return l_memdup(s, sizeof(*s));
+}
+
 /*
  * Build a scalar = value modulo p where p is the prime number for a given
- * curve.  bytes can contain a numer with up to 2x number of digits as the
+ * curve.  bytes can contain a number with up to 2x number of digits as the
  * curve.  This is used in Hash to Curve calculations.
  */
 LIB_EXPORT struct l_ecc_scalar *l_ecc_scalar_new_modp(
@@ -830,6 +884,41 @@ LIB_EXPORT struct l_ecc_scalar *l_ecc_scalar_new_modp(
 	return NULL;
 }
 
+LIB_EXPORT struct l_ecc_scalar *l_ecc_scalar_new_modn(
+					const struct l_ecc_curve *curve,
+					const void *bytes, size_t len)
+{
+	struct l_ecc_scalar *c;
+	uint64_t tmp[2 * L_ECC_MAX_DIGITS];
+	unsigned int ndigits = len / 8;
+
+	if (!bytes)
+		return NULL;
+
+	if (len % 8)
+		return NULL;
+
+	if (ndigits > curve->ndigits * 2)
+		return NULL;
+
+	c = _ecc_constant_new(curve, NULL, 0);
+	if (!c)
+		return NULL;
+
+	memset(tmp, 0, sizeof(tmp));
+	_ecc_be2native(tmp, bytes, ndigits);
+
+	_vli_mmod_slow(c->c, tmp, curve->n, curve->ndigits);
+
+	if (!_vli_is_zero_or_one(c->c, curve->ndigits) &&
+			secure_memcmp_64(curve->n, c->c, curve->ndigits) > 0)
+		return c;
+
+	l_ecc_scalar_free(c);
+
+	return NULL;
+}
+
 /*
  * Takes a buffer of the same size as the curve and scales it to a range
  * 1..n using value = (value mod (n - 1)) + 1.  For the curves we support
@@ -924,6 +1013,18 @@ LIB_EXPORT bool l_ecc_point_multiply(struct l_ecc_point *ret,
 	return true;
 }
 
+LIB_EXPORT bool l_ecc_point_multiply_g(struct l_ecc_point *ret,
+					const struct l_ecc_scalar *scalar)
+{
+	if (unlikely(!ret || !scalar))
+		return false;
+
+	_ecc_point_mult(ret, &scalar->curve->g, scalar->c, NULL,
+						scalar->curve->p);
+
+	return true;
+}
+
 LIB_EXPORT bool l_ecc_point_add(struct l_ecc_point *ret,
 					const struct l_ecc_point *a,
 					const struct l_ecc_point *b)
@@ -995,3 +1096,8 @@ LIB_EXPORT bool l_ecc_points_are_equal(const struct l_ecc_point *a,
 	return ((memcmp(a->x, b->x, a->curve->ndigits * 8) == 0) &&
 			(memcmp(a->y, b->y, a->curve->ndigits * 8) == 0));
 }
+
+LIB_EXPORT bool l_ecc_point_is_infinity(const struct l_ecc_point *p)
+{
+	return _ecc_point_is_zero(p);
+}
